@@ -19,7 +19,12 @@ class Namespace(object):
 
     def read_attribute(self, name):
         try:
-            return getattr(self, name)
+            if hasattr(self, '__dict__') and self.__dict__ and name in self.__dict__:
+                return self.__dict__[name]
+            if hasattr(self, '__slots__') and self.__slots__ and name in self.__slots__:
+                return self.__slots__[name]
+
+            raise RookAttributeNotFound(name)
         except AttributeError:
             raise RookAttributeNotFound(name)
 
