@@ -54,6 +54,5 @@ try:
         def __call__(self, *args, **kwargs):
             return super(RookoutChalice, self).__call__(*args, **kwargs)
 
-
-except ImportError:
+except ImportError: # If Chalice is not availale, we just won't have a wrapper for it
     pass
