@@ -9,7 +9,6 @@ import site
 import imp
 import hashlib
 
-from rook.services.exts.cloud_debug_python.module_explorer import _GetLineNumbers
 from six.moves import builtins
 
 from rook.logger import logger
@@ -70,17 +69,6 @@ class ImportService(object):
 
     NAME = "Import"
 
-    class Notification(object):
-        def __init__(self, module_name, module_filename, include_externals, lineno, callback, removed, aug, file_hash):
-            self.module_name = module_name
-            self.module_filename = module_filename
-            self.callback = callback
-            self.removed = removed
-            self.include_externals = include_externals
-            self.lineno = lineno
-            self.aug = aug
-            self.file_hash = file_hash
-
     def __init__(self, bdb_location_service):
         self._bdb_location_service = bdb_location_service
 
@@ -93,13 +81,32 @@ class ImportService(object):
 
         self.file_utils = FileUtils()
 
-        external_paths = [sys.exec_prefix]
-        if hasattr(site, 'getsitepackages'):
-            external_paths = external_paths + site.getsitepackages()
+        external_paths = [sys.prefix]
+
+        if hasattr(sys, 'base_prefix') and sys.base_prefix not in external_paths:
+            external_paths += [sys.base_prefix]
+
+        if hasattr(site, 'getsitepackages') and hasattr(site, 'getsitepackages') and hasattr(site, 'getusersitepackages'):
+            external_paths += site.getsitepackages() + [site.getuserbase() + site.getusersitepackages()]
 
         self.external_paths = [os.path.normcase(os.path.realpath(external_path))
                                for external_path in external_paths]
 
+        # pyenv messes up sys.path, so we ignore everything under it's folder
+        for path in self.external_paths:
+            pyenv_offset = path.find('.pyenv')
+            if pyenv_offset != -1:
+                pyenv_directory = path[:pyenv_offset + len('.pyenv') + 1]
+                self.external_paths += [pyenv_directory]
+                break
+
+        # If we are (heuristicly) worried we missed anything, add "catch-alls"
+        if len(self.external_paths) < 3:
+            logger.debug("Missing getsitepackages, adding heuristics")
+            self.external_paths += ['site-packages', 'site-python']
+
+        logger.debug("External paths are " + str(self.external_paths))
+
         if on_lambda():
             _BLACKLISTED_PATHS.extend(['/var/task/flask', '/var/task/chalice'])
 
@@ -178,24 +185,20 @@ class ImportService(object):
 
             self._thread.join()
 
-    def register_post_import_notification(self, aug, name, filepath, aug_file_hash, include_externals, lineno, callback, removed):
-        # Normalize file path
-        if filepath:
-            filepath = os.path.normcase(os.path.normpath(filepath))
-
+    def register_post_import_notification(self, location):
+        filepath = location.filename
         filename = os.path.basename(filepath) if filepath else None
-        notification = self.Notification(name, filepath, include_externals, lineno, callback, removed, aug, aug_file_hash)
 
         # Register notification for future loads
-        self._post_import_notifications[aug.aug_id] = notification
+        self._post_import_notifications[location.aug_id] = location
 
         # declaring filenames blacklist for avoiding setting multiple callbacks on the same aug
         filenames_blacklist = []
         match_found = False
         file_hashes = {}
         with CountingImportLock():
-            # Attempt to satisfy notification using known modules
-            for module_name, module_object in six.iteritems(self._modules):
+            # Attempt to satisfy location using known modules
+            for module_object in six.itervalues(self._modules):
                 # Get module details and check if it matches
                 module_filename = self._get_module_path(module_object)
 
@@ -211,21 +214,21 @@ class ImportService(object):
 
                 if module_filename:
                     if module_filename not in filenames_blacklist and self._does_module_match_notification(
-                            module_object.__name__, module_filename, notification, self.external_paths):
+                            module_filename, location, self.external_paths):
                         match_found = True
                         filenames_blacklist.append(module_filename)
-                        notification.callback(module_object)
+                        location.module_found(module_object)
 
         if not match_found:
-            if aug_file_hash in file_hashes:
-                aug.send_warning(Error(exc=RookSourceFilePathSuggestion(filepath, file_hashes[aug_file_hash])))
+            if location.file_hash in file_hashes:
+                location.send_warning(Error(exc=RookSourceFilePathSuggestion(filepath, file_hashes[location.file_hash])))
 
     def remove_aug(self, aug_id):
-        notification = self._post_import_notifications.pop(aug_id, None)
-        if notification is None:
+        location = self._post_import_notifications.pop(aug_id, None)
+        if location is None:
             return
 
-        notification.removed()
+        location.set_removed()
 
     def clear_augs(self):
         # This does not require a lock - `notifications` may be added to or removed from in a different thread
@@ -236,8 +239,8 @@ class ImportService(object):
         # At that point, it's safe to iterate over `notifications` and remove all notifications.
         notifications = self._post_import_notifications
         self._post_import_notifications = dict()
-        for notification in six.itervalues(notifications):
-            notification.removed()
+        for location in six.itervalues(notifications):
+            location.set_removed()
 
     def pre_fork(self):
         CountingImportLock().__enter__()
@@ -292,7 +295,7 @@ class ImportService(object):
             for module_name, module in six.iteritems(modules):
                 module_filename = self._get_module_path(module)
                 if module_name not in old_modules.keys() and self._is_valid_module(module, module_filename):
-                    self._notify_of_new_module(module.__name__, module, module_filename)
+                    self._notify_of_new_module(module, module_filename)
 
             # Update the "old" list
             self._modules = modules
@@ -300,24 +303,24 @@ class ImportService(object):
         except Exception:
             logger.exception("Exception in ImportService")
 
-    def _notify_of_new_module(self, module_name, module_object, module_filename):
-        self._trigger_all_notifications_for_module(module_filename, module_name, module_object)
+    def _notify_of_new_module(self, module_object, module_filename):
+        self._trigger_all_notifications_for_module(module_filename, module_object)
 
-    def _trigger_all_notifications_for_module(self, module_filename, module_name, module):
+    def _trigger_all_notifications_for_module(self, module_filename, module):
         if module_filename:
-            for aug_id, notification in six.iteritems(self._post_import_notifications.copy()):
-                if self._does_module_match_notification(module_name, module_filename, notification,
+            for aug_id, location in six.iteritems(self._post_import_notifications.copy()):
+                if self._does_module_match_notification(module_filename, location,
                                                         self.external_paths):
                     try:
-                        notification.callback(module)
+                        location.module_found(module)
                     except:  # lgtm[py/catch-base-exception]
                         logger.exception("Error on module load callback")
-                elif os.path.basename(notification.module_filename) == os.path.basename(module_filename):
+                elif os.path.basename(location.filename) == os.path.basename(module_filename):
                     file_content = self.file_utils.get_safe_file_contents(module)
                     if file_content:
                         file_hash = hashlib.sha256(file_content).hexdigest()
-                        if notification.file_hash == file_hash:
-                            notification.aug.send_warning(Error(exc=RookSourceFilePathSuggestion(notification.module_filename, module_filename)))
+                        if location.file_hash == file_hash:
+                            location.send_warning(Error(exc=RookSourceFilePathSuggestion(location.filename, module_filename)))
 
     def _get_module_path(self, module):
         if module is None or not hasattr(module, '__name__') or not hasattr(module, '__file__'):
@@ -337,7 +340,7 @@ class ImportService(object):
                 return None
 
             if path:
-                result = os.path.realpath(os.path.normcase(os.path.abspath(path)))
+                result = os.path.normcase(os.path.abspath(path))
                 self._path_cache[module.__name__] = result
                 return result
 
@@ -351,23 +354,18 @@ class ImportService(object):
         return False
 
     @staticmethod
-    def _does_module_match_notification(module_name, module_filename, notification, external_paths):
-        if not notification.include_externals:
+    def _does_module_match_notification(module_filename, location, external_paths):
+        if not location.include_externals:
             for external_path in external_paths:
-                if module_filename.startswith(external_path):
+                if external_path in module_filename:
                     return False
 
-        if notification.module_filename and ImportService.path_contains_path(module_filename, notification.module_filename) or \
-                        notification.module_name and module_name == notification.module_name and \
+        if location.filename and ImportService.path_contains_path(module_filename, location.filename) and \
                         not ImportService.is_black_listed_path(module_filename):
             return True
         else:
             return False
 
-    @staticmethod
-    def _does_code_object_match_notification(code_object, notification):
-        return notification.lineno in _GetLineNumbers(code_object)
-
     @staticmethod
     def path_contains_path(full_path, partial_path):
         if full_path.endswith(partial_path):
