@@ -11,7 +11,7 @@ import uuid
 # This must be the first import, otherwise enable_gevent_for_grpc will fail since grpc has already been imported
 from rook.logger import logger
 
-from rook.config import AgentComConfiguration
+from rook.config import AgentComConfiguration, ShutdownConfig
 
 from rook.com_ws.output_ws import Output
 
@@ -22,6 +22,7 @@ from .services.monitor.monitor_service import MonitorService
 from rook.exceptions import RookInterfaceException, RookVersionNotSupported
 from rook.com_ws.agent_com_ws import AgentCom, thread_connection_type, SINGLE_THREAD_AGENT_COM
 from rook.com_ws.command_handler import CommandHandler
+from rook.serverless import on_lambda
 
 try:
     from .atfork import install_fork_handler, remove_fork_handler  # lgtm[py/cyclic-import]
@@ -63,7 +64,7 @@ class _Singleton(object):
         self._init_id()
         self._output = Output(self._id)
 
-        if _enable_monitor:
+        if _enable_monitor and not on_lambda:
             self._monitor_service = MonitorService.get_instance()
             self._monitor_service.start()
 
@@ -98,6 +99,10 @@ class _Singleton(object):
         if install_fork_handler:
             install_fork_handler(fork)
 
+        install_fault_handler = debug or os.environ.get('ROOKOUT_ENABLE_SEGFAULT')
+        if install_fault_handler is not None and install_fault_handler == '1':
+            self._install_fault_handler()
+
         logger.debug("Initiating AgentCom-\t%s:%d", host, int(port))
 
         labels = labels or {}
@@ -168,7 +173,7 @@ class _Singleton(object):
     def _check_version_supported():
         try:
             supported_platforms = ['pypy', 'cpython']
-            supported_version = ['2.7', '3.5', '3.6', '3.7', '3.8']
+            supported_version = ['2.7', '3.5', '3.6', '3.7', '3.8', '3.9']
 
             current_platform = platform.python_implementation().lower()
             if current_platform not in supported_platforms:
@@ -185,19 +190,34 @@ class _Singleton(object):
 
             raise e
 
+    @staticmethod
+    def _install_fault_handler():
+        major, _, _, _, _ = sys.version_info
+        if major == 3:
+            try:
+                import faulthandler
+                faulthandler.enable(file=sys.stderr, all_threads=True)
+            except: # lgtm[py/catch-base-exception]
+                logger.exception("Failed to install fault handler")
+
 
 singleton_obj = _Singleton()
 
 
 def exit_handler():
     try:
-        logger.info("Exit handler called - flushing and closing WebSocket")
-        if singleton_obj._agent_com:
-            singleton_obj.flush()
-            singleton_obj._agent_com.stop()
+        try:
+            ShutdownConfig.IS_SHUTTING_DOWN = True
+
+            logger.info("Exit handler called - flushing and closing WebSocket")
+            if singleton_obj._agent_com:
+                singleton_obj.flush()
+                singleton_obj._agent_com.stop()
+        except:  # lgtm[py/catch-base-exception]
+            logger.exception("Flush and close failed")
+        logger.info("Exit handler finished")
     except:  # lgtm[py/catch-base-exception]
-        logger.exception("Flush and close failed")
-    logger.info("Exit handler finished")
+        pass
 
 
 atexit.register(exit_handler)
