@@ -30,3 +30,8 @@ class UserWarnings(object):
     def send_warning(cls, error):
         if hasattr(cls._tls, "warning_reporter") and cls._tls.warning_reporter:
             cls._tls.warning_reporter.send_warning(error)
+
+    @classmethod
+    def set_error(cls, error):
+        if hasattr(cls._tls, "warning_reporter") and cls._tls.warning_reporter:
+            cls._tls.warning_reporter.set_error(error)
